{
******************************************************
*                                                    *
*    Dev-IDE source code (Dev-C++/Dev-Pascal)        *
*                for Linux/Qt                        *
*                                                    *
*  Author : Colin Laplace                            *
*                                                    *
*  compoptions.pas : Compiler options window         *
*                                                    *
******************************************************
}

unit cpp_compoptions;

interface

uses
  SysUtils, Classes, QDialogs, QForms,
  QStdCtrls, Inifiles, QExtCtrls, QComCtrls,
  QControls, QButtons, version;

type
  TCpp_CompForm = class(TForm)
    OkBtn: TBitBtn;
    CancelBtn: TBitBtn;
    DefaultBtn: TBitBtn;
    PageControl: TPageControl;
    CTab: TTabSheet;
    Panel1: TPanel;
    CodeGenTab: TTabSheet;
    LinkerTab: TTabSheet;
    DirTab: TTabSheet;
    Panel3: TPanel;
    Panel4: TPanel;
    ObjC: TCheckBox;
    DebugInfo: TCheckBox;
    NoLibs: TCheckBox;
    Panel6: TPanel;
    ComCheck: TCheckBox;
    Commands: TEdit;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    eCdir: TEdit;
    eCppDir: TEdit;
    eLibDir: TEdit;
    GroupBox1: TGroupBox;
    Ansi: TCheckBox;
    Traditional: TCheckBox;
    Warning: TCheckBox;
    GroupBox2: TGroupBox;
    AccessChecking: TCheckBox;
    AcceptDollar: TCheckBox;
    Heuristics: TCheckBox;
    GroupBox3: TGroupBox;
    Optimize: TCheckBox;
    Best: TCheckBox;
    MinorOpt: TCheckBox;
    GroupBox4: TGroupBox;
    Exceptions: TCheckBox;
    DoubleFloat: TCheckBox;
    Memory: TCheckBox;
    Panel2: TPanel;
    Label6: TLabel;
    Label5: TLabel;
    Label7: TLabel;
    HelpBtn: TBitBtn;
    MakeLabel: TLabel;
    MakeEdit: TEdit;
    GccEdit: TEdit;
    Label1: TLabel;
    Label8: TLabel;
    GppEdit: TEdit;
    MakeBtn: TSpeedButton;
    GccBtn: TSpeedButton;
    GppBtn: TSpeedButton;
    OpenDialog: TOpenDialog;
    procedure CancelBtnClick(Sender: TObject);
    procedure OkBtnClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure DefaultBtnClick(Sender: TObject);
    procedure MakeBtnClick(Sender: TObject);
    procedure GccBtnClick(Sender: TObject);
    procedure GppBtnClick(Sender: TObject);
  private
    { Dclarations prives }
  public
    { Dclarations publiques }
  end;

var
  NumOpt : integer;

implementation
uses Main, dev_module;
{$R *.xfm}

procedure TCpp_CompForm.CancelBtnClick(Sender: TObject);
begin
  Close;
end;

procedure TCpp_CompForm.OkBtnClick(Sender: TObject);
begin
  MainForm.CompilerOptionsModified := True;
  if ComCheck.Checked and (Commands.Text = '') then
     begin
          MessageDlg('You have selected the "Add commands" option but no commands have been typed in the field below.', MtError,[MbOK],0);
          ComCheck.Checked := False;
     end;
  { Write to the ini file the user's desired compiler options }
  with MainForm do begin
  IniFile.WriteBool('C_Compiler','Ansi',Ansi.Checked);
  IniFile.WriteBool('C_Compiler','Traditional',Traditional.Checked);
  IniFile.WriteBool('C_Compiler','Warning',Warning.Checked);

  IniFile.WriteBool('C++_Compiler','AccessChecking',AccessChecking.Checked);
  IniFile.WriteBool('C++_Compiler','AcceptDollar',AcceptDollar.Checked);
  IniFile.WriteBool('C++_Compiler','Heuristics',Heuristics.Checked);

  IniFile.WriteBool('Code_Generation','Exceptions',Exceptions.Checked);
  IniFile.WriteBool('Code_Generation','DoubleFloat',DoubleFloat.Checked);
  IniFile.WriteBool('Code_Generation','Memory',Memory.Checked);

  IniFile.WriteBool('Linker','ObjC',ObjC.Checked);
  IniFile.WriteBool('Linker','DebugInfo',DebugInfo.Checked);
  IniFile.WriteBool('Linker','NoLibs',NoLibs.Checked);

  IniFile.WriteBool('Optimization','Optimize',Optimize.Checked);
  IniFile.WriteBool('Optimization','Best',Best.Checked);
  IniFile.WriteBool('Optimization','MinorOpt',MinorOpt.Checked);

  if ComCheck.Checked then
     begin
       IniFile.WriteBool('Directories','AddCommands',ComCheck.Checked);
       IniFile.WriteString('Directories','Commands',Commands.Text);
     end
  else
     begin
       IniFile.WriteBool('Directories','AddCommands',False);
       IniFile.WriteString('Directories','Commands','');
     end;
  IniFile.WriteString('dev','make_dir',MakeEdit.Text);
  IniFile.WriteString('dev','gcc_dir',GccEdit.Text);
  IniFile.WriteString('dev','g++_dir',GppEdit.Text);
  Module.Make := MakeEdit.Text;
  Module.Compiler := GccEdit.Text;
  Module.CppCompiler := GppEdit.Text;
  
  IniFile.WriteString('Directories','CDir',eCDir.Text);
  IniFile.WriteString('Directories','CppDir',eCppDir.Text);
  IniFile.WriteString('Directories','LibDir',eLibDir.Text);
  end;
end;

procedure TCpp_CompForm.FormActivate(Sender: TObject);
begin
   { Read from the ini file the user's desired compiler options }
  with MainForm do begin
  Ansi.Checked := IniFile.ReadBool('C_Compiler','Ansi',Ansi.Checked);
  Traditional.Checked := IniFile.ReadBool('C_Compiler','Traditional',Traditional.Checked);
  Warning.Checked := IniFile.ReadBool('C_Compiler','Warning',Warning.Checked);

  AccessChecking.Checked := IniFile.ReadBool('C++_Compiler','AccessChecking',AccessChecking.Checked);
  AcceptDollar.Checked := IniFile.ReadBool('C++_Compiler','AcceptDollar',AcceptDollar.Checked);
  Heuristics.Checked := IniFile.ReadBool('C++_Compiler','Heuristics',Heuristics.Checked);

  Exceptions.Checked := IniFile.ReadBool('Code_Generation','Exceptions',Exceptions.Checked);
  DoubleFloat.Checked := IniFile.ReadBool('Code_Generation','DoubleFloat',DoubleFloat.Checked);
  Memory.Checked := IniFile.ReadBool('Code_Generation','Memory',Memory.Checked);

  ObjC.Checked := IniFile.ReadBool('Linker','ObjC',ObjC.Checked);
  DebugInfo.Checked := IniFile.ReadBool('Linker','DebugInfo',DebugInfo.Checked);
  NoLibs.Checked := IniFile.ReadBool('Linker','NoLibs',NoLibs.Checked);

  Optimize.Checked := IniFile.ReadBool('Optimization','Optimize',Optimize.Checked);
  Best.Checked := IniFile.ReadBool('Optimization','Best',Best.Checked);
  MinorOpt.Checked := IniFile.ReadBool('Optimization','MinorOpt',MinorOpt.Checked);

  ComCheck.Checked := IniFile.ReadBool('Directories','AddCommands',ComCheck.Checked);
  Commands.Text := IniFile.ReadString('Directories','Commands',Commands.Text);

  MakeEdit.Text := IniFile.ReadString('dev', 'make_dir', MAKE);
  GccEdit.Text := IniFile.ReadString('dev', 'gcc_dir', GCC);
  GppEdit.Text := IniFile.ReadString('dev', 'g++_dir', GPP);
  eCDir.Text := IniFile.ReadString('Directories','CDir',eCDir.Text);
  eCppDir.Text := IniFile.ReadString('Directories','CppDir',eCppDir.Text);
  eLibDir.Text := IniFile.ReadString('Directories','LibDir',eLibDir.Text);
  end;
end;

procedure TCpp_CompForm.DefaultBtnClick(Sender: TObject);
begin
{ Set default compiler options }

  Ansi.Checked := false;
  Traditional.Checked := false;
  Warning.Checked := false;

  AccessChecking.Checked := false;
  AcceptDollar.Checked := true;
  Heuristics.Checked := true;

  Exceptions.Checked := false;
  DoubleFloat.Checked := false;
  Memory.Checked := false;

  ObjC.Checked := false;
  DebugInfo.Checked := false;
  NoLibs.Checked := false;

  Optimize.Checked := false;
  Best.Checked := false;
  MinorOpt.Checked := true;
  ComCheck.Checked := False;
  Commands.Text := '';

  MakeEdit.Text := MAKE;
  GccEdit.Text := GCC;
  GppEdit.Text := GPP;
end;

procedure TCpp_CompForm.MakeBtnClick(Sender: TObject);
begin
  if OpenDialog.Execute then
    MakeEdit.Text := OpenDialog.FileName;
end;

procedure TCpp_CompForm.GccBtnClick(Sender: TObject);
begin
  if OpenDialog.Execute then
    GccEdit.Text := OpenDialog.FileName;
end;

procedure TCpp_CompForm.GppBtnClick(Sender: TObject);
begin
  if OpenDialog.Execute then
    GppEdit.Text := OpenDialog.FileName;
end;


end.
