{
******************************************************
*                                                    *
*    Dev-IDE source code (Dev-C++/Dev-Pascal)        *
*                for Linux/Qt                        *
*                                                    *
*  Author : Colin Laplace                            *
*                                                    *
*  editor.pas : source file tabsheet object          *
*                                                    *
******************************************************
}
unit editor;

interface

uses Qt, SysUtils, Types, Classes, Variants, QGraphics, QControls, QForms, QDialogs,
  QTypes, QMenus, QImgList, QComCtrls, QStdCtrls, QExtCtrls, SynEdit,SynEditKeyCmds;

type
  TEditor = class(TTabSheet)
  procedure Init(In_Project : boolean; Caption_, File_name : string; DoOpen : boolean);
  destructor Free;
  function FindReplace : boolean;
  procedure Find;
  procedure Replace;
  procedure EditorKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
  procedure EditorKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);

  private

  public
    InProject : boolean;
    Filename  : string;
    New       : boolean;
    Modified  : boolean;
    Text : TSynEdit;
  end;

implementation

uses main, project, dev_module;

procedure TEditor.Init(In_Project : boolean; Caption_, File_name : string; DoOpen : boolean);
begin
  InProject := In_Project;
  Caption := Caption_;
  FileName := File_name;
  Visible := True;
  PageControl := MainForm.PageControl;
  PageControl.ActivePage := self;
  Text := TSynEdit.Create(self);
  Text.Parent := self;
  Text.Align := alClient;
  Text.Visible := True;
  if (DoOpen) then
    try
      Text.Lines.LoadFromFile(FileName);
      New := False;
    except
      MessageDlg('Could not open file "'+FileName+'"', mtError, [mbOK], 0);
    end
  else
    New := True;
  InsertControl(Text);
  Text.PopupMenu := MainForm.EditPopupMenu;
  Text.Font.Name := 'courier';
  Text.Font.Size := 10;
  Text.Color := clWhite;
  Text.WantTabs := True;
  Text.OnKeyDown := EditorKeyDown;
  Text.OnKeyUp := EditorKeyUp;
  Text.Highlighter := Module.Syn;
  Application.ProcessMessages;
end;

destructor TEditor.Free;
begin
//  Text.Free;
 inherited Free;
end;

function TEditor.FindReplace : boolean;
var rOptions: TSynSearchOptions;
    sSearch: string;
begin
  sSearch := MainForm.ReplaceDialog.FindText;
  Result := True;
  if Length(sSearch) = 0 then begin
    MessageDlg('Cannot search for empty text',MtWarning,[MbOK],0);
    MainForm.FindNextItem.Enabled := False;
  end else begin
    rOptions := [];
    if not (frDown in MainForm.ReplaceDialog.Options) then
      Include(rOptions, ssoBackwards);
    if frMatchCase in MainForm.ReplaceDialog.Options then
      Include(rOptions, ssoMatchCase);
    if frWholeWord in MainForm.ReplaceDialog.Options then
      Include(rOptions, ssoWholeWord);
    if Text.SearchReplace(sSearch, '', rOptions) = 0 then begin
      if not (frReplaceAll in MainForm.ReplaceDialog.Options) then
         MessageDlg('Search text ''' + sSearch + ''' not found', MtInformation, [mbOK],0);
      Result := False;
    end
  end;
end;

procedure TEditor.Find;
var rOptions: TSynSearchOptions;
    sSearch: string;
begin
  sSearch := MainForm.FindDialog.FindText;
  if Length(sSearch) = 0 then begin
    MessageDlg('Cannot search for empty text',MtWarning,[MbOK],0);
    MainForm.FindNextItem.Enabled := False;
  end else begin
    rOptions := [];
    if not (frDown in MainForm.FindDialog.Options) then
      Include(rOptions, ssoBackwards);
    if frMatchCase in MainForm.FindDialog.Options then
      Include(rOptions, ssoMatchCase);
    if frWholeWord in MainForm.FindDialog.Options then
      Include(rOptions, ssoWholeWord);
    if Text.SearchReplace(sSearch, 'df', rOptions) = 0 then begin
      MessageDlg('Search text ''' + sSearch + ''' not found', MtInformation, [mbOK],0);
      MainForm.FindNextItem.Enabled := False;
    end else
      MainForm.FindNextItem.Enabled := True;
  end;
end;

procedure TEditor.Replace;
begin
  if frReplaceAll in MainForm.ReplaceDialog.Options then begin
     while FindReplace do
           Text.SelText := MainForm.ReplaceDialog.ReplaceText;
  end else if Text.SelText <> '' then
     Text.SelText := MainForm.ReplaceDialog.ReplaceText;
end;

procedure TEditor.EditorKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  case Key of
  Key_Insert : if (not (ssCtrl in Shift)) and (not (ssShift in Shift)) then  begin
                if MainForm.StatusBar.Panels[2].Text = 'Insertion' then
                   MainForm.StatusBar.Panels[2].Text := 'Replacement'
                else
                   MainForm.StatusBar.Panels[2].Text := 'Insertion';
                Exit;
              end;
  Key_End, Key_Home, Key_Left, Key_Up, Key_Right, Key_Down,
  Key_Shift, Key_Control, Key_NumLock, Key_Escape, Key_Menu,
  Key_Prior, Key_Next, Key_Pause : exit;
  end;
  //if (ssCtrl in Shift) or (ssAlt in Shift) then
  //   exit;
  Modified := True;
  MainForm.StatusBar.Panels[1].Text := 'Modified';
  MainForm.StatusBar.Panels[0].Text := '  '+IntToStr(Text.CaretX)+': '+IntToStr(Text.CaretY);
  MainForm.StatusBar.Panels[3].Text := IntToStr(Text.Lines.Count) + ' lines in file';
end;

procedure TEditor.EditorKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  MainForm.StatusBar.Panels[0].Text := '  '+IntToStr(Text.CaretX)+': '+IntToStr(Text.CaretY);
  MainForm.StatusBar.Panels[3].Text := IntToStr(Text.Lines.Count) + ' lines in file';
end;


end.
