{
******************************************************
*                                                    *
*    Dev-IDE source code (Dev-C++/Dev-Pascal)        *
*                for Linux/Qt                        *
*                                                    *
*  Author : Colin Laplace                            *
*                                                    *
*  enviropt.pas : Environment options window         *
*                                                    *
******************************************************
}

unit enviropt;

interface

uses
  version, SysUtils, Classes, QForms, QDialogs, IniFiles, QMask,
  QStdCtrls, QComCtrls, QControls, QButtons, dev_module, SynEdit, QGraphics, Utils,
  SynCompletionProposal;

type
  TEnvirForm = class(TForm)
    OkBtn: TBitBtn;
    CancelBtn: TBitBtn;
    DefaultBtn: TBitBtn;
    PageControl: TPageControl;
    Preferences: TTabSheet;
    GroupBox2: TGroupBox;
    SavePos: TCheckBox;
    ColorSheet: TTabSheet;
    DefaultDirectory: TGroupBox;
    Defaultdir: TEdit;
    ExecutionPanel: TGroupBox;
    Minimize: TCheckBox;
    Windows: TGroupBox;
    TabSheet1: TTabSheet;
    ElementList: TListBox;
    BitBtn2: TBitBtn;
    NewConfBtn: TBitBtn;
    Backup: TCheckBox;
    TabSheet2: TTabSheet;
    GroupBox5: TGroupBox;
    GroupBox1: TGroupBox;
    SizeLabel: TLabel;
    FontLabel: TLabel;
    Font: TComboBox;
    Size: TEdit;
    GroupBox3: TGroupBox;
    EditorColor: TComboBox;
    BackupPanel: TGroupBox;
    LineNumbers: TCheckBox;
    GroupBox7: TGroupBox;
    ShowScrollHint: TCheckBox;
    DontShowScroll: TCheckBox;
    NoProjExpl: TCheckBox;
    ExecParam: TCheckBox;
    Params: TEdit;
    TabIndent: TSpinEdit;
    AutoIndent: TCheckBox;
    Label1: TLabel;
    CodeSheet: TTabSheet;
    GroupBox6: TGroupBox;
    Label2: TLabel;
    GroupBox9: TGroupBox;
    Memo: TMemo;
    TestBtn: TBitBtn;
    ToolBarSave: TCheckBox;
    CompileStayOnTop: TCheckBox;
    TabToSpace: TCheckBox;
    SmartTabs: TCheckBox;
    ShowGutter: TCheckBox;
    HelpBtn: TBitBtn;
    ExeInDir: TCheckBox;
    OpenAll: TCheckBox;
    GroupBox4: TGroupBox;
    Bold: TCheckBox;
    Italic: TCheckBox;
    Label3: TLabel;
    TermEdit: TEdit;
    Label4: TLabel;
    GdbEdit: TEdit;
    Underlined: TCheckBox;
    Label5: TLabel;
    Coloredit: TEdit;
    SelectBtn: TBitBtn;
    ColorDialog: TColorDialog;
    SynEdit: TSynEdit;
    Complet: TSynCompletionProposal;
    procedure OkBtnClick(Sender: TObject);
    procedure CancelBtnClick(Sender: TObject);
    procedure DefaultBtnClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure ResetDirsClick(Sender: TObject);
    procedure CreateIniClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FontChange(Sender: TObject);
    procedure ElementListClick(Sender: TObject);
    procedure ColorChange(Color : integer);
    procedure UpdateFontStyles(Sender: TObject);
    procedure TestBtnClick(Sender: TObject);
    procedure AutoIndentClick(Sender: TObject);
    procedure LoadFontStyles(Sender: TObject);
    procedure SelectBtnClick(Sender: TObject);
  private
    { Dclarations prives }
  public
    { Dclarations publiques }
  end;

var
  EnvirForm: TEnvirForm;

implementation

uses Main, Editor;

{$R *.xfm}

procedure TEnvirForm.OkBtnClick(Sender: TObject);
var
  TrueSize : integer;

   function MakeCommaText(Fore, Back: Integer; Bold, Italic, Underline:Boolean): string;
   begin
      Result:= Format('%d, %d, %d, %d, %d', [Fore, Back, Ord(Bold), Ord(Italic), Ord(Underline)]);
   end;
begin
  TrueSize := StrToInt(Size.Text);
  if (TrueSize > 100) or (TrueSize < 4) then
     begin
          MessageDlg('Size must be between 4 and 100',MtError,[MbOK],0);
          Exit;
     end;

  if DefaultDir.Text <> '' then
     with MainForm do begin
       OpenFileDialog.InitialDir := DefaultDir.Text;
       OpenAddDialog.InitialDir  := DefaultDir.Text;
       SaveProject.InitialDir  := DefaultDir.Text;
       ExportDialog.InitialDir := DefaultDir.Text;
    end;

  { Write to the ini file the user's desired environment options }

with MainForm.IniFile do begin
  WriteString('Options','DefaultDir',Defaultdir.text);
  if EditorColor.Text<>'' then
     WriteString('Options','Color', EditorColor.Text);

  WriteBool('Options','SavePos', SavePos.Checked);
  WriteString('Options','Font', Font.Text);
  WriteInteger('Options','FontSize', StrToInt(Size.Text));
  WriteBool('Options','Backup',Backup.Checked);
  WriteBool('Options','Minimize',Minimize.Checked);
  WriteBool('Options','LineNumbers',LineNumbers.Checked);
  WriteBool('Options','ShowScrollHint',ShowScrollHint.Checked);
  WriteBool('Options','DontShowScroll',DontShowScroll.Checked);
  WriteBool('Options','NoProjExpl',NoProjExpl.Checked);
  MainForm.ProjectView.Visible := not NoProjExpl.Checked;
  WriteBool('Options','ToolbarSave',ToolbarSave.Checked);
  WriteBool('Options','CompileStayOnTop', CompileStayOnTop.Checked);
  WriteBool('Options','OpenAll', OpenAll.Checked);
  WriteBool('Options','ExecParam',ExecParam.Checked);
  WriteString('Options','Params', Params.Text);
  WriteBool('Options','AutoIndent',AutoIndent.Checked);
  WriteBool('Options','TabsToSpaces',TabToSpace.Checked);
  WriteBool('Options','ShowGutter',ShowGutter.Checked);
  WriteBool('Options','SmartTabs',SmartTabs.Checked);
  WriteBool('Options','ExeInDir',ExeInDir.Checked);
  WriteString('dev', 'term_dir', TermEdit.Text);
  WriteString('dev', 'gdb_dir', GdbEdit.Text);
  try
    Memo.Lines.SaveToFile(Module.ConfDir + DEV_COMPLETION_FILE);
  except
    MessageDlg('Could not save Code Completion list file.', mtError, [mbOK], 0);
  end;

{  try
    DefaultSourceCode.Lines.SaveToFile(MainForm.BinDir+DEFAULTCODE_FILE)
  except
    MessageDlg('Could not save Default code file.', mtError, [mbOK], 0);
  end;}

  WriteInteger('Options','TabIndent',TabIndent.Value);

  with Module.Syn do begin
    WriteString('EditorColor','CommentAttri',
                 MakeCommaText(CommentAttri.Foreground, CommentAttri.Background,
                               fsBold in CommentAttri.Style, fsItalic in CommentAttri.Style,
                               fsUnderline in CommentAttri.Style ));
    WriteString('EditorColor','IdentifierAttri',
                 MakeCommaText(IdentifierAttri.Foreground, IdentifierAttri.Background,
                               fsBold in IdentifierAttri.Style, fsItalic in IdentifierAttri.Style,
                               fsUnderline in IdentifierAttri.Style ));
    WriteString('EditorColor','KeyAttri',
                 MakeCommaText(KeyAttri.Foreground, KeyAttri.Background,
                               fsBold in KeyAttri.Style, fsItalic in KeyAttri.Style,
                               fsUnderline in KeyAttri.Style ));
    WriteString('EditorColor','NumberAttri',
                 MakeCommaText(NumberAttri.Foreground, NumberAttri.Background,
                               fsBold in NumberAttri.Style, fsItalic in NumberAttri.Style,
                               fsUnderline in NumberAttri.Style ));
    WriteString('EditorColor','SpaceAttri',
                 MakeCommaText(SpaceAttri.Foreground, SpaceAttri.Background,
                               fsBold in SpaceAttri.Style, fsItalic in SpaceAttri.Style,
                               fsUnderline in SpaceAttri.Style ));
    WriteString('EditorColor','StringAttri',
                 MakeCommaText(StringAttri.Foreground, StringAttri.Background,
                               fsBold in StringAttri.Style, fsItalic in StringAttri.Style,
                               fsUnderline in StringAttri.Style ));
    WriteString('EditorColor','SymbolAttri',
                 MakeCommaText(SymbolAttri.Foreground, SymbolAttri.Background,
                               fsBold in SymbolAttri.Style, fsItalic in SymbolAttri.Style,
                               fsUnderline in SymbolAttri.Style ));
    WriteString('EditorColor','DirecAttri',
                MakeCommaText(DirecAttri.Foreground, DirecAttri.Background,
                              fsBold in DirecAttri.Style, fsItalic in DirecAttri.Style,
                              fsUnderline in DirecAttri.Style ));
  end;
  UpdateFile;
  end;
end;

procedure TEnvirForm.CancelBtnClick(Sender: TObject);
begin
  Close;
end;

procedure TEnvirForm.DefaultBtnClick(Sender: TObject);
begin
  { Set environment default options }
  LineNumbers.Checked := False;
  DontShowScroll.Checked := False;
  ShowScrollHint.Checked := True;
  DefaultDir.Text:= '';
  Params.Text := '';
  EditorColor.Text:='White';
  Font.Text := 'Courier New';
  Size.Text := '10';
  SavePos.Checked := True;
  Backup.Checked := false;
  Minimize.Checked := True;
  TabIndent.Value := 4;
  ExecParam.Checked := False;
  NoProjExpl.Checked := False;
  AutoIndent.Checked := True;
  ToolBarSave.Checked := True;
  CompileStayOnTop.Checked := False;
  TabToSpace.Checked := True;
  ShowGutter.Checked := True;
  SmartTabs.Checked := True;
  ExeInDir.Checked := True;
  OpenAll.Checked := False;
  TermEdit.Text := XTERM;
  Module.CreateNewCompletionFile;
  Memo.Lines.LoadFromFile(Module.ConfDir + DEV_COMPLETION_FILE);
//  MainForm.CreateNewDefaultFile;
//  DefaultSourceCode.Lines.LoadFromFile(MainForm.DevPath+DEFAULTCODE_FILE);
end;

procedure TEnvirForm.FormActivate(Sender: TObject);
var s : string;
begin
  with MainForm.IniFile do begin
  { Read from the ini file the user's desired environment options }

  Defaultdir.Text:=ReadString('Options','DefaultDir',Defaultdir.text);

  s := ReadString('Options','Color', 'White');

  if s = 'White' then
     EditorColor.ItemIndex := 0
  else if s = 'Yellow' then
     EditorColor.ItemIndex := 1
  else if s = 'Blue' then
     EditorColor.ItemIndex := 2
  else if s = 'Aqua' then
     EditorColor.ItemIndex := 3;

  SavePos.Checked := ReadBool('Options','SavePos',SavePos.Checked);
  Backup.Checked := ReadBool('Options','Backup',Backup.Checked);
  Minimize.Checked := ReadBool('Options','Minimize',Minimize.Checked);
  Font.Text := ReadString('Options','Font', Font.Text);
  Size.Text := IntToStr(ReadInteger('Options','FontSize', StrToInt(Size.Text)));
  TabIndent.Value := ReadInteger('Options','TabIndent',4);
  LineNumbers.Checked := ReadBool('Options','LineNumbers',False);
  ShowScrollHint.Checked := ReadBool('Options','ShowScrollHint',True);
  DontShowScroll.Checked := ReadBool('Options','DontShowScroll',False);
  NoProjExpl.Checked := ReadBool('Options','NoProjExpl',False);
  ToolBarSave.Checked := ReadBool('Options','ToolbarSave',True);
  CompileStayOnTop.Checked := ReadBool('Options','CompileStayOnTop', False);
  OpenAll.Checked := ReadBool('Options','OpenAll', False);

  ExecParam.Checked := ReadBool('Options','ExecParam',False);
  Params.Text := ReadString('Options','Params', '');

  AutoIndent.Checked := ReadBool('Options','AutoIndent',True);
  ShowGutter.Checked := ReadBool('Options','ShowGutter',True);
  SmartTabs.Checked := ReadBool('Options','SmartTabs',True);
  ExeInDir.Checked := ReadBool('Options','ExeInDir',True);

  if AutoIndent.Checked then
     TabIndent.Enabled := False
  else TabIndent.Enabled := True;

  TabToSpace.Checked := ReadBool('Options','TabsToSpaces',True);
  Termedit.Text := ReadString('dev', 'term_dir', XTERM);
  Termedit.Text := ReadString('dev', 'gdb_dir', GDB);

  Synedit.Highlighter := Module.Syn;
  end;

{  if not FileExists(MainForm.DevPath+COMPLETION_FILE) then
     MainForm.CreateNewCompletionFile;
  Memo.Lines.LoadFromFile(MainForm.DevPath+COMPLETION_FILE);}


{  if not FileExists(MainForm.DevPath+DEFAULTCODE_FILE) then
     MainForm.CreateNewDefaultFile;
  DefaultSourceCode.Lines.LoadFromFile(MainForm.BinDir+DEFAULTCODE_FILE)}
end;

procedure TEnvirForm.ResetDirsClick(Sender: TObject);
begin
  MainForm.IniFile.WriteString('dev', 'make_dir', MAKE);
  Module.Make := MAKE;
  MainForm.IniFile.WriteString('dev', 'term_dir', XTERM);
  Module.Term := XTERM;
  TermEdit.Text := XTERM;
  MainForm.IniFile.WriteString('dev', 'gdb_dir', GDB);
  Module.Gdb := GDB;
  GdbEdit.Text := GDB;
  {$IFDEF CPP}
  MainForm.IniFile.WriteString('dev', 'gcc_dir', GCC);
  Module.Compiler := GCC;
  MainForm.IniFile.WriteString('dev', 'g++_dir', GPP);
  Module.CppCompiler := GPP;
  {$ELSE}
  MainForm.IniFile.WriteString('dev', 'ppc_dir', PPC386);
  Module.Compiler := PPC386;
  {$ENDIF}
end;

procedure TEnvirForm.CreateIniClick(Sender: TObject);
begin
//  MainForm.CreateNewIniFile(false);
end;

procedure TEnvirForm.FormCreate(Sender: TObject);
begin
  Font.Items.AddStrings(Screen.Fonts);
end;

procedure TEnvirForm.FontChange(Sender: TObject);
begin
  if Font.ItemIndex = 4 then
     Font.ItemIndex := 3;
end;

procedure TEnvirForm.ElementListClick(Sender: TObject);
begin
  with MainForm.IniFile do begin
    case ElementList.ItemIndex of
      0 : ColorEdit.Text := IntToHex(ReadInteger('Colors','Comment',0), 6);
      1 : ColorEdit.Text := IntToHex(ReadInteger('Colors','Identifier',0), 6);
      2 : ColorEdit.Text := IntToHex(ReadInteger('Colors','Key',0), 6);
      3 : ColorEdit.Text := IntToHex(ReadInteger('Colors','Number',0), 6);
      4 : ColorEdit.Text := IntToHex(ReadInteger('Colors','Space',0), 6);
      5 : ColorEdit.Text := IntToHex(ReadInteger('Colors','String',0), 6);
      6 : ColorEdit.Text := IntToHex(ReadInteger('Colors','Symbol',0), 6);
      7 : ColorEdit.Text := IntToHex(ReadInteger('Colors','Color',0), 6);
      8 : ColorEdit.Text := IntToHex(ReadInteger('Colors','Directives', 0), 6);
    end;
    LoadFontStyles(sender);
  end;
end;

procedure TEnvirForm.ColorChange(Color : integer);
begin
  with MainForm.IniFile do begin
    case ElementList.ItemIndex of
      0 :
      begin
        Module.Syn.CommentAttri.Foreground := Color;
        WriteInteger('Colors', 'Comment', Color);
      end;
      1 :
      begin
        Module.Syn.IdentifierAttri.Foreground := Color;
        WriteInteger('Colors', 'Identifier', Color);
      end;
      2 :
      begin
        Module.Syn.KeyAttri.Foreground := Color;
        WriteInteger('Colors', 'Key', Color);
      end;
      3 :
      begin
        Module.Syn.NumberAttri.Foreground := Color;
        WriteInteger('Colors', 'Number', Color);
      end;
      4 :
      begin
        Module.Syn.SpaceAttri.Foreground := Color;
        WriteInteger('Colors', 'Space', Color);
      end;
      5 :
      begin
        Module.Syn.StringAttri.Foreground := Color;
        WriteInteger('Colors', 'String', Color);
      end;
      6 :
      begin
        Module.Syn.SymbolAttri.Foreground := Color;
        WriteInteger('Colors', 'Symbol', Color);
      end;
      7 :
      begin
        SynEdit.Font.Color := Color;
        WriteInteger('Colors', 'Color', Color);
      end;
      8 :
      begin
        Module.Syn.DirecAttri.Foreground := Color;
        WriteInteger('Colors', 'Directives', Color);
      end;
   end;
 end;
end;

procedure TEnvirForm.LoadFontStyles(Sender: TObject);
begin
 case ElementList.ItemIndex of
   0 : with Module.Syn.CommentAttri do begin
         if fsBold in Style then
           Bold.Checked := True
         else Bold.Checked := False;
         if fsItalic in Style then
           Italic.Checked := True
         else Italic.Checked := False;
         if fsUnderline in Style then
           Underlined.Checked := True
         else Underlined.Checked := False;
       end;
   1 : with Module.Syn.IdentifierAttri do begin
         if fsBold in Style then
            Bold.Checked := True
         else Bold.Checked := False;
         if fsItalic in Style then
            Italic.Checked := True
         else Italic.Checked := False;
         if fsUnderline in Style then
            Underlined.Checked := True
         else Underlined.Checked := False;
       end;
   2 : with Module.Syn.KeyAttri do begin
         if fsBold in Style then
            Bold.Checked := True
         else Bold.Checked := False;
         if fsItalic in Style then
            Italic.Checked := True
         else Italic.Checked := False;
        if fsUnderline in Style then
           Underlined.Checked := True
        else Underlined.Checked := False;
       end;
   3 : with Module.Syn.NumberAttri do begin
         if fsBold in Style then
            Bold.Checked := True
         else Bold.Checked := False;
         if fsItalic in Style then
            Italic.Checked := True
         else Italic.Checked := False;
         if fsUnderline in Style then
           Underlined.Checked := True
         else Underlined.Checked := False;
       end;
   4 : with Module.Syn.SpaceAttri do begin
         if fsBold in Style then
            Bold.Checked := True
         else Bold.Checked := False;
         if fsItalic in Style then
            Italic.Checked := True
         else Italic.Checked := False;
         if fsUnderline in Style then
           Underlined.Checked := True
         else Underlined.Checked := False;
       end;
   5 : with Module.Syn.StringAttri do begin
         if fsBold in Style then
            Bold.Checked := True
         else Bold.Checked := False;
         if fsItalic in Style then
            Italic.Checked := True
         else Italic.Checked := False;
         if fsUnderline in Style then
           Underlined.Checked := True
         else Underlined.Checked := False;
       end;
   6 : with Module.Syn.SymbolAttri do begin
         if fsBold in Style then
            Bold.Checked := True
         else Bold.Checked := False;
         if fsItalic in Style then
            Italic.Checked := True
         else Italic.Checked := False;
         if fsUnderline in Style then
           Underlined.Checked := True
         else Underlined.Checked := False;
       end;
   end;
end;

procedure TEnvirForm.UpdateFontStyles(Sender: TObject);
var ChkBox: TCheckBox;
begin
   ChkBox := TCheckBox(Sender);
   case ElementList.ItemIndex of
     0 : with Module.Syn.CommentAttri do
           case chkBox.Tag of
             0 : if chkBox.Checked then
                    Style:= Style + [fsBold]
                 else
                    Style:= Style - [fsBold];
             1 : if chkBox.Checked then
                    Style:= Style + [fsItalic]
                 else
                    Style:= Style - [fsItalic];
             2 : if chkBox.Checked then
                    Style:= Style + [fsUnderline]
                 else
                    Style:= Style - [fsUnderline];
           end;
     1 : with Module.Syn.IdentifierAttri do
           case chkBox.Tag of
             0 : if chkBox.Checked then
                    Style:= Style + [fsBold]
                 else
                    Style:= Style - [fsBold];
             1 : if chkBox.Checked then
                    Style:= Style + [fsItalic]
                 else
                    Style:= Style - [fsItalic];
             2 : if chkBox.Checked then
                    Style:= Style + [fsUnderline]
                 else
                    Style:= Style - [fsUnderline];
           end;
      2 : with Module.Syn.KeyAttri do
            case chkBox.Tag of
              0 : if chkBox.Checked then
                     Style:= Style + [fsBold]
                   else
                     Style:= Style - [fsBold];
              1 : if chkBox.Checked then
                     Style:= Style + [fsItalic]
                  else
                     Style:= Style - [fsItalic];
              2 : if chkBox.Checked then
                     Style:= Style + [fsUnderline]
                  else
                     Style:= Style - [fsUnderline];
            end;
      3 : with Module.Syn.NumberAttri do
            case chkBox.Tag of
              0 : if chkBox.Checked then
                     Style:= Style + [fsBold]
                  else
                     Style:= Style - [fsBold];
              1 : if chkBox.Checked then
                     Style:= Style + [fsItalic]
                  else
                     Style:= Style - [fsItalic];
              2 : if chkBox.Checked then
                     Style:= Style + [fsUnderline]
                  else
                     Style:= Style - [fsUnderline];
            end;
      4 : with Module.Syn.SpaceAttri do
            case chkBox.Tag of
              0 : if chkBox.Checked then
                     Style:= Style + [fsBold]
                  else
                     Style:= Style - [fsBold];
              1 : if chkBox.Checked then
                     Style:= Style + [fsItalic]
                  else
                     Style:= Style - [fsItalic];
              2 : if chkBox.Checked then
                     Style:= Style + [fsUnderline]
                  else
                     Style:= Style - [fsUnderline];
            end;
      5 : with Module.Syn.StringAttri do
            case chkBox.Tag of
              0 : if chkBox.Checked then
                     Style:= Style + [fsBold]
                  else
                     Style:= Style - [fsBold];
              1 : if chkBox.Checked then
                     Style:= Style + [fsItalic]
                  else
                     Style:= Style - [fsItalic];
              2 : if chkBox.Checked then
                     Style:= Style + [fsUnderline]
                  else
                     Style:= Style - [fsUnderline];
            end;
      6 : with Module.Syn.SymbolAttri do
            case chkBox.Tag of
              0 : if chkBox.Checked then
                     Style:= Style + [fsBold]
                  else
                     Style:= Style - [fsBold];
              1 : if chkBox.Checked then
                     Style:= Style + [fsItalic]
                  else
                     Style:= Style - [fsItalic];
              2 : if chkBox.Checked then
                     Style:= Style + [fsUnderline]
                  else
                     Style:= Style - [fsUnderline];
            end;
  end;
end;

procedure TEnvirForm.TestBtnClick(Sender: TObject);
begin
  Complet.ItemList.Assign(Memo.Lines);
  Complet.Execute('', Self.Left+TestBtn.Left+TestBtn.Width, Self.Top + TestBtn.Top+TestBtn.Height);
end;

procedure TEnvirForm.AutoIndentClick(Sender: TObject);
begin
  if AutoIndent.Checked then
     TabIndent.Enabled := False
  else TabIndent.Enabled := True;
end;
 procedure TEnvirForm.SelectBtnClick(Sender: TObject);
begin
   if ColorDialog.Execute then begin
     ColorEdit.Text := IntToHex(ColorDialog.Color, 6);
     ColorChange(ColorDialog.Color);
   end;
end;


end.
