{
******************************************************
*                                                    *
*    Dev-IDE source code (Dev-C++/Dev-Pascal)        *
*                for Linux/Qt                        *
*                                                    *
*  Author : Colin Laplace                            *
*                                                    *
*  newproject.pas : New project window               *
*                                                    *
******************************************************
}

unit newproject;

interface

uses
  SysUtils, Types, Classes, Variants, QGraphics, QControls, QForms, QDialogs,
  QComCtrls, QImgList, QStdCtrls, QButtons, QExtCtrls, IniFiles, Qt;

type
  TTemplate = Class
  public
    FileName: String;
    Name, Icon, Description, Catagory: String;
    Text, Text_Cpp: String;
    EnableC, EnableCpp: Boolean;
    CursorX, CursorY, CursorX_Cpp, CursorY_Cpp: Integer;
    Console, Cpp, DLL : Boolean;
    CompilerOptions, Libs, ProjectName, IncludeDirs: String;
  end;
  
  TNewProjForm = class(TForm)
    PageControl: TPageControl;
    ProjectSheet: TTabSheet;
    Label1: TLabel;
    OkBtn: TBitBtn;
    CancelBtn: TBitBtn;
    CppPanel: TPanel;
    CBtn: TRadioButton;
    CppBtn: TRadioButton;
    Default: TCheckBox;
    NameLabel: TLabel;
    NameEdit: TEdit;
    ProjView: TIconView;
    templatelabel: TLabel;
    TemplatesImageList: TImageList;
    ImageList: TImageList;
    procedure FormActivate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ReadTemplateIndex;
    procedure ListViewChange(Sender: TObject; Item: TIconViewItem; Selected: Boolean);
    procedure ListViewDblClick(sender : TObject; Item: TIconViewItem);
    procedure ProjViewSelectItem(Sender: TObject; Item: TIconViewItem;
      Selected: Boolean);
    procedure CustomKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
  public
    Templates: TList;
    { Public declarations }
  end;
var
  NewProjForm: TNewProjForm;

implementation
uses dev_module;

{$R *.xfm}

procedure TNewProjForm.FormActivate(Sender: TObject);
begin
{$IFDEF CPP}
  CppPanel.Visible := true;
{$ENDIF}
end;

procedure TNewProjForm.FormCreate(Sender: TObject);
begin
  ReadTemplateIndex;
end;

procedure TNewProjForm.ReadTemplateIndex;
var
  TemplateFiles: TStringList;

procedure AddTemplate(AFileName: String);
var
  Template: TTemplate;
  Ini: TIniFile;
begin
  Ini := TIniFile.Create(AFileName);
  Ini.CaseSensitive := false;
  Template := TTemplate.Create;
  with Template do
  with Ini do
  begin
     Template.FileName := AFileName;
     Name := ReadString('Template', 'Name', 'Template');
     Icon := Module.TemplateDir + ReadString('Template', 'Icon', '');
     Description := ReadString('Template', 'Description', 'No description available.');
     Catagory := ReadString('Template', 'Catagory', 'Additional');
     Text := ReadString('Editor', 'Text', ChangeFileExt(ExtractFileName(AFileName), '_c.txt'));
     Text := Module.TemplateDir + '/' + Text;
     Text_Cpp := ReadString('Editor', 'Text_Cpp', ChangeFileExt(ExtractFileName(AFileName), '_cpp.txt'));
     Text_Cpp := Module.TemplateDir + '/' + Text_Cpp;
     CursorX := ReadInteger('Editor', 'CursorX', 1);
     CursorY := ReadInteger('Editor', 'CursorY', 1);
     CursorX_Cpp := ReadInteger('Editor', 'CursorX_Cpp', 1);
     CursorY_Cpp := ReadInteger('Editor', 'CursorY_Cpp', 1);
     Console := ReadBool('Project', 'Console', True);
     Cpp := ReadBool('Project', 'Cpp', True);
     EnableC := ReadBool('Template', 'EnableC', not Cpp);
     EnableCpp := ReadBool('Template', 'EnableCpp', Cpp);
     Dll := ReadBool('Project', 'Dll', False);
     CompilerOptions := ReadString('Project', 'CompilerOptions', '');
     IncludeDirs := ReadString('Project', 'IncludeDirs', '');
     Libs := ReadString('Project', 'Libs', '');
     ProjectName := ReadString('Project', 'Name', '');
  end;
  Templates.Add(Template);
  Ini.Free;
end;

procedure SortCatagories;
var
  Catagories: TStringList;
  Template: TTemplate;
  i: Integer;
  Tab: TTabSheet;
  ListView: TIconView;
begin
  Catagories := TStringList.Create;
  for i := 0 to Templates.Count - 1 do
  begin
     Template := Templates.Items[i];
     if Catagories.IndexOf(Template.Catagory) = -1 then begin
        Catagories.Add(Template.Catagory);
        Tab := TTabSheet.Create(Self);
        Tab.Caption := Catagories.Strings[i];
        Tab.PageControl := PageControl;
        ListView := TIconView.Create(Tab);
        ListView.Align := alClient;
        Tab.InsertControl(ListView);
        ListView.Images := TemplatesImageList;
        ListView.OnSelectItem := ListViewChange;
        ListView.OnItemDoubleClick := ListViewDblClick;
        ListView.OnKeyDown := CustomKeyDown;
        ListView.Visible := True;
     end
  end;
  Catagories.Free;
end;

procedure SortTemplates;
var
  i, j : Integer;
  ListView : TIconView;
  Item : TIconViewItem;
  Template: TTemplate;
begin
  for i := 0 to Templates.Count - 1 do
  begin
     Template := Templates.Items[i];
     for j := 1 to PageControl.PageCount - 1 do
        if PageControl.Pages[j].Caption = Template.Catagory then
        begin
           ListView := TIconView(PageControl.Pages[j].Controls[0]);
           Item := TIconViewItem(ListView.Items.Add);
           Item.Caption := Template.Name;
           Item.AllowRename := False;
           Item.ImageIndex := 0;
        end;
  end;
end;

procedure FilesFromWildcard(Directory, Mask: String;
  var Files : TStringList; Subdirs, ShowDirs, Multitasking: Boolean);
var
  SearchRec: TSearchRec;
  Attr, Error: Integer;
begin
  { First, find the required file... }
  Attr := faAnyFile;
  if ShowDirs = False then
     Attr := Attr - faDirectory;
  Error := FindFirst(Directory + Mask, Attr, SearchRec);
  if (Error = 0) then
  begin
     while (Error = 0) do
     begin
     { Found one! }
        Files.Add(Directory + SearchRec.Name);
        Error := FindNext(SearchRec);
        if Multitasking then
           Application.ProcessMessages;
     end;
     FindClose(SearchRec);
  end;

  { Then walk through all subdirectories. }
  if Subdirs then
  begin
     Error := FindFirst(Directory + '*.*', faAnyFile, SearchRec);
     if (Error = 0) then
     begin
        while (Error = 0) do
        begin
           { Found one! }
           if (SearchRec.Name[1] <> '.') and (SearchRec.Attr and
             faDirectory <> 0) then
              { We do this recursively! }
              FilesFromWildcard(Directory + SearchRec.Name, Mask, Files,
                Subdirs, ShowDirs, Multitasking);
           Error := FindNext(SearchRec);
        end;
     FindClose(SearchRec);
     end;
  end;
end;

var
  i: Integer;
begin
  TemplateFiles := TStringList.Create;
  FilesFromWildcard(Module.TemplateDir +'/', '*.template', TemplateFiles, False,
    False, True);
  if TemplateFiles.Count > 0 then
  begin
     Templates := TList.Create;
     for i := 0 to TemplateFiles.Count - 1 do
        AddTemplate(TemplateFiles.Strings[i]);
     SortCatagories;
     SortTemplates;
  end;
  TemplateFiles.Free;
end;

procedure TNewProjForm.ListViewChange(Sender: TObject;
  Item: TIconViewItem; Selected: Boolean);
var
  Template: TTemplate;
  i : integer;
  ok : boolean;
begin
  ok := false;
  Template := nil;
  for i := 0 to Templates.Count - 1 do begin
    Template := Templates.Items[i];
    if (Template.Name = Item.Caption) then begin
      ok := true;
      break;
    end;
  end;
  if not ok then exit;
  TemplateLabel.Caption := Template.Description;
  {$IFDEF CPP}
  if Template.Cpp then
  begin
    CppBtn.Checked := True;
    CBtn.Checked := False;
  end else begin
    CppBtn.Checked := False;
    CBtn.Checked := True;
  end;
  {$ENDIF}
end;

procedure TNewProjForm.ListViewDblClick(Sender: TObject; Item : TIconViewItem);
begin
  ModalResult := mrOk;
  close;
end;

procedure TNewProjForm.ProjViewSelectItem(Sender: TObject;
  Item: TIconViewItem; Selected: Boolean);
begin
  case ProjView.Selected.Index of
  0: TemplateLabel.Caption := 'A standard console application';
  1: TemplateLabel.Caption := 'A standard GUI application';
  2: TemplateLabel.Caption := 'A Static library (.a)';
  3: TemplateLabel.Caption := 'A Dynamic library (shared object .so)';
  4: TemplateLabel.Caption := 'An empty project';
  end;
end;

procedure TNewProjForm.CustomKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = Key_delete then
end;

end.
