{
******************************************************
*                                                    *
*    Dev-IDE source code (Dev-C++/Dev-Pascal)        *
*                for Linux/Qt                        *
*                                                    *
*  Author : Colin Laplace                            *
*                                                    *
*  utils.pas : various functions, most are           *   
*              Linux/Unix routines using the libC    *
*                                                    *
******************************************************
}

{$WARN SYMBOL_PLATFORM OFF}

unit utils;

interface
uses Libc, QDialogs, version, QGraphics, SynEditHighlighter, SysUtils;

procedure GetStyles(Attri : TSynHighlighterAttributes; AttriName : string);
function MakeCommaTextToColor(Text:string; Index:Integer; DefaultColor: TColor): TColor;
function MakeCommaTextToBool(Text:string; Index:Integer; Default : boolean): boolean;
procedure ExecuteFile(filename, argus : string; wait : boolean);
procedure ExecuteRedirFile(filename, argus, redirname : string; wait : boolean);
procedure PutXMode(s : string);
procedure handle_segfault;

const MODE755 = S_IRUSR or S_IWUSR or S_IXUSR or S_IRGRP or S_IXGRP or S_IROTH or S_IXOTH;
const MODE644 = S_IRUSR or S_IWUSR or S_IRGRP or S_IROTH;

implementation
uses main;

procedure GetStyles(Attri : TSynHighlighterAttributes; AttriName : string);
var Default : boolean;
begin
  with Attri do begin
    if AttriName = 'KeyAttri' then
      Default := True
    else
      Default := False;
    if MakeCommaTextToBool(MainForm.IniFile.ReadString('EditorColor',AttriName,''), 2, False) then
       Attri.Style := Attri.Style + [fsBold]
    else Attri.Style := Attri.Style - [fsBold];
    if MakeCommaTextToBool(MainForm.IniFile.ReadString('EditorColor',AttriName,''), 3, False) then
       Attri.Style := Attri.Style + [fsItalic]
    else Attri.Style := Attri.Style - [fsItalic];
  end;
end;

function  GetCommaText(const Text:string; Index:Integer; Comma: Char): string;
var
  I, Pos1, Width, Temp, Temp1: Integer;
  S: string;
begin
   Result:= Text;
   if Text = '' then Exit;
   S:= Text;
   Pos1:= 0;
   Temp1 := 0;
   for I:= 0 to Index do
   begin
       Temp:= Pos(Comma, S);
       if Temp = 0 then Break;
       Temp1:= Pos1;
       Pos1:= Pos1 + Temp;
       S:= Copy(Text, Pos1 + 1, Length(S));
   end;
   Width:= Pos1 - Temp1 -1;
   Result:= Trim(Copy(Text, Temp1 + 1, Width));
end;


function MakeCommaTextToColor(Text:string; Index:Integer; DefaultColor: TColor): TColor;
var
   S: string;
begin
   Result:= DefaultColor;
   if Text = '' then Exit;
   S:= GetCommaText(Text, Index, ',');
   Result:= StrToInt(S);
end;

function MakeCommaTextToBool(Text:string; Index:Integer; Default : boolean): boolean;
var
   S: string;
begin
   Result:= Default;
   if Text = '' then Exit;
   S := GetCommaText(Text, Index, ',');
   Result := S = '1';;
end;

function count_item(s : string) : integer;
var i : integer;
begin
  result := 0;
  i := 0;
  while (i <> length(s) - 1) do
    if s[i] = ' ' then begin
      inc(result);
      inc(i);
      while (s[i] <> #0) and (s[i] = ' ') do
        inc(i);
    end
    else
      inc(i);
  inc(result);
end;

type TPPChar = array [0..256] of PChar;

procedure ExecuteFile(filename, argus : string; wait : boolean);
var PP : TPPchar;
    pid : integer;
    count : integer;
    i, j, k : integer;
    str : pchar;
begin
  if FileName = '' then begin
    Messagedlg('No program to execute. Be sure an xterm is correctly set in '+
               'Environment Options, Misc sheet. If this doesn''t work please '+
               'verify your compiler and make paths in Compiler Options', mtError, [mbOK], 0);
    exit;
  end;
  count := count_item(FileName + ' ' +argus);
  GetMem(str, length(FileName + ' ' +argus) + 1);
  strcpy(str, pchar(FileName + ' ' +argus));
  str[length(pchar(FileName + ' ' +argus))] := #0;
  for i := 0 to count - 1 do begin
    j := 0;
    while (str[j] <> #0) and (str[j] <> ' ') do
      inc(j);
    GetMem(PP[i], j);
    for k := 0 to j - 1 do
      PP[i][k] := str[k];
    PP[i][j] := #0;
    str := str + j + 1;
  end;
  PP[count] := nil;
  pid := fork;
  if pid = 0 then begin
    if execve(pchar(Filename), @pp, envp) <= 0 then begin
      MessageDlg('Cound not execute '+filename, mtError, [mbOK], 0);
      exit;
    end;
  end
  else if wait then
    waitpid(pid, nil, 0);
{  for i := 0 to count - 1 do begin
    writeln(i);
    free(pp[i]);
  end;}
end;

procedure ExecuteRedirFile(filename, argus, redirname : string; wait : boolean);
var f : integer;
    PP : TPPchar;
    pid : integer;
    count : integer;
    i, j, k : integer;
    str : pchar;
begin
  if FileName = '' then begin
    Messagedlg('No program to execute. Be sure the compiler paths are correctly set in Compiler Options', mtError, [mbOK], 0);
    exit;
  end;
  count := count_item(FileName + ' ' +argus);
  GetMem(str, length(FileName + ' ' +argus) + 1);
  strcpy(str, pchar(FileName + ' ' +argus));
  str[length(pchar(FileName + ' ' +argus))] := #0;
  for i := 0 to count - 1 do begin
    j := 0;
    while (str[j] <> #0) and (str[j] <> ' ') do
      inc(j);
    GetMem(PP[i], j);
    for k := 0 to j - 1 do
      PP[i][k] := str[k];
    PP[i][j] := #0;
    str := str + j + 1;
  end;
  PP[count] := nil;
  f := open(pchar(redirname), O_CREAT or O_RDWR or O_TRUNC, MODE644);
  if f < 0 then begin
     writeln('Could not log to file '+redirname);
     exit;
  end;
  pid := fork();
  if (pid <> 0) and wait then
    waitpid(pid, nil, 0)
  else begin
    {$IFDEF CPP}
    __close(stderr);
    dup2(f, stderr);
    {$ELSE}
    __close(1);
    dup2(f, 1);
    {$ENDIF}
    if execve(pchar(Filename), @pp, envp) <= 0 then begin
      MessageDlg('Cound not execute '+filename+'. Check directory settings in Compiler Options', mtError, [mbOK], 0);
      exit;
    end;
  end;
//  for i := 0 to count - 1 do
//    free(pp[i]);
  __close(f);
end;

procedure PutXMode(s : string);
begin
  chmod(pchar(s), MODE755);
end;

var seg : TSignalHandler;
var child : TSignalHandler;

procedure handle_segfault;
var c : char;
begin
  writeln('SEGMENTATION FAULT OCCURED !!!'+#10+
  'Please email haiku@bloodshed.net about this bug, with a little explanation '+
  'on how the bug appeard. Please also include the version number : '+ DEV_VERSION);
  writeln;
  writeln('The program can try to save your modified files before the crash append');
  writeln('Do you want to try a "Save all" ? (y/n)');
  read(c);
  if c = 'y' then
    MainForm.SaveallItemClick(MainForm);
  signal(SIGSEGV, nil);
  seg := nil;
end;

procedure handle_sigchild;
begin
  if getpgrp() <> 0 then
    waitpid(0, nil, 0);
end;

begin
  child := @handle_sigchild;
  signal(SIGCHLD, child);
{$IFDEF RELEASE}
  seg := @handle_segfault;
  signal(SIGSEGV, seg);
{$ENDIF}
end.
